/* see usage() function for version and script description */

var DEBUG = 0;  // 0 = release, 1 = debug, optionally pass setting to script as argv[MINARGS + 1]

var S_OK = 0, S_FALSE = 1, E_ABORT = 2147483655, E_FAIL = 2147500037, 
	MINARGS = 0;  // MINARGS = number of cli arguments necessary for script to do its job

var oArgV = WScript.Arguments, iArgC = oArgV.Count();  
if (MINARGS > iArgC) { var hr = usage(); WScript.Quit(1); }

var ir = main(iArgC, oArgV);
WScript.Quit(ir);


function main(iArgC, oArgV) {
	var hr = S_OK;
	try {
		// validate passed parameters
		if ((0 > iArgC) || !oArgV) { hr = E_FAIL; throw hr; }

		for (var iIndex = 0; iIndex < iArgC; iIndex++) {
			switch (oArgV.item(iIndex)) {
			case "/h" || "/?":
				/* or
				var reSwitches = /\/[h?]/gi,  // regular expression pattern for debug switches
				var br = reSwitches.test(oArgV.item(iIndex));  // test for occurrences of regular expression
				if (br) { var hr = usage(); WScript.Quit(1); }
				*/
				var hr = usage(); WScript.Quit(1);
				break;
			case "/d":
				/* or
				var reSwitches = /[0-1]/g,  // regular expression pattern for debug switches
				var br = reSwitches.test(oArgV.item(iIndex));  // test for occurrences of regular expression
				if (br) DEBUG = 0;
				*/
				DEBUG = 1;
				break;
			default:
				// do nothing
				break;
			}
		}

		if (-1 == DEBUG) debugger;

		hr = FireEvents();
		if (hr) throw hr;
	}

	catch(oError) {
		if (isNaN(oError) /* or "number" == typeOf(oError) */) {
			WScript.Echo("main() exception handler caught an exception with error number " + oError.number.toString() + " with description " + oError.description + ".", 0, "DebugMessage");
			hr = oError.number; 
		}
		else {
			WScript.Echo("main() exception handler caught an exception with error number " + hr.toString(), 0, "DebugMessage");
		}
	}

	if (1 == DEBUG) debugger;

	return hr;
}


function usage() {
	var hr = S_OK;
	try {
		if (-1 == DEBUG) debugger;
		WScript.Echo("version = 05jan01\n");
		WScript.Echo("description = tests the general purpose event provider service\n");
		WScript.Echo("usage = test.js [/h] [/d]\n");
		WScript.Echo("where");
		WScript.Echo("  /h = display help");
		WScript.Echo("  /d = stop at hardcoded debugger breakpoints");
	}

	catch(oError) {
		if (-1 == DEBUG) debugger;
		if (isNaN(oError)) { var sTemp = oError.description; hr = oError.number; }
	}

	if (-1 == DEBUG)	debugger;
	
	return hr;
}


function FireEvents() {
	var hr = S_OK;
	try {
		// gpep generic event, create time and hostname get autogenerated and written with record
		if (1 == DEBUG) debugger;
		var oGpep = new ActiveXObject("GnlPpsEvntPvdr.Gpep");
		oGpep.Fire("myApp.js", "billyBob@mydomain.com", 1694, "c:\\ops\\myApp.js");
		
		// gpep create event
		if (1 == DEBUG) debugger;
		// var oMsi = new ActiveXObject("WindowsInstaller.Installer");
		// vHostname = oMsi.Environment("computername");
		vHostname = oGpep.LocalHostname();
		// vCreationDate = CurrentWmiDateTime(), 
		vCreationDate = oGpep.CurrentWmiDateTime();	
		oGpep.Fire("myOpsScript.js", "joeBob@hosterxyz.com", 1666, "c:\\ops\\myOpsScript.js", vHostname,
                    vCreationDate)

		// now wait for a few seconds to simulate application usage duration
		if (1 == DEBUG) debugger;
		var dtStart = new Date(), dtCurrent = new Date(dtStart), iWaitInSeconds = 10;
		var iTimeSpan = Math.abs(dtCurrent.getSeconds() - dtStart.getSeconds());
		while (iTimeSpan < iWaitInSeconds) {
			dtCurrent = new Date();
			iTimeSpan = Math.abs(dtCurrent.getSeconds() - dtStart.getSeconds());
		}

		// gpep terminate event
		if (1 == DEBUG) debugger;
	    vTerminationDate = oGpep.CurrentWmiDateTime();
		oGpep.Fire("myOpsScript.vbp", "joebob@hosterxyz.com", 1666, "c:\\ops\\myOpsScript.vbp", vHostname,
					vCreationDate, vTerminationDate)
		if (1 == DEBUG) debugger;
	}

	catch(oError) {
		if (-1 == DEBUG) debugger;
		if (isNaN(oError)) { var sTemp = oError.description; hr = oError.number; }	
	}
	
	return hr;
}


// Implement as IGpep:GetWmiDateTime([in, out, retval] bsWmiDateTime) so you don't have to carry
// around all these wmi date time support functions for every application language scenario

function CurrentWmiDateTime() {
	// convert local systemtime structure "ddd mmm dd hh:mm:ss dst yyyy" to wmi datetime format "yyyymmddHHMMSS.mmmmmmsUUU"
	// e.g.    Wed Jan 10 00:43:38 PST 2001 to 20000501040800.000340-420 | -480 for pst depending on time of year
	var hr = S_OK;
	try {
		if (-1 == DEBUG) debugger;
	
		var oLocalSystemTime = new Date();
	    // var sWmiDateTime = "00000000000000.000000-000";
		var sWmiDateTime = oLocalSystemTime.getFullYear().toString();

		if (10 > (oLocalSystemTime.getMonth() + 1)) sWmiDateTime += "0";
		sWmiDateTime += (oLocalSystemTime.getMonth() + 1).toString();  // zero based representation of months

		if (10 > oLocalSystemTime.getDate()) sWmiDateTime += "0";
		sWmiDateTime += oLocalSystemTime.getDate().toString();  // getDay() is for day of week

		if (10 > oLocalSystemTime.getHours()) sWmiDateTime += "0";
		sWmiDateTime += oLocalSystemTime.getHours().toString();

		if (10 > oLocalSystemTime.getMinutes()) sWmiDateTime += "0";
		sWmiDateTime += oLocalSystemTime.getMinutes().toString();

		if (10 > oLocalSystemTime.getSeconds()) sWmiDateTime += "0";
		sWmiDateTime += oLocalSystemTime.getSeconds().toString();

		sWmiDateTime += ".";
		
		var lTemp = oLocalSystemTime.getMilliseconds() * 1000;
		for (iPow = 1; iPow < 6; iPow++)
			if (Math.pow(10, iPow) > lTemp) sWmiDateTime += "0";
		sWmiDateTime += lTemp.toString();

		if (-1 == DEBUG) debugger;
		lTemp = oLocalSystemTime.getTimezoneOffset();
		if (0 > lTemp) {  // the offset result signs are opposite what you might expect
			lTemp *= -1;
			sWmiDateTime += "+";
		}
		else {
			sWmiDateTime += "-";
		}
		if (10 > lTemp) sWmiDateTime += "0";
		if (100 > lTemp) sWmiDateTime += "0";
		sWmiDateTime += lTemp.toString();

		if (-1 == DEBUG) debugger;
	}

	catch(oError) {
		if (-1 == DEBUG) debugger;
		if (isNaN(oError)) { var sTemp = oError.description; hr = oError.number; }	
	}
	
	// return hr;
	return sWmiDateTime
}

function SmallDateTimeFormat(bsWmiDtFmt)
{
	// convert yyyymmddHHMMSS.mmmmmmsUUU to yyyy-mm-dd hh:mm:ss
	// e.g.    20000501040800.000340-420 to 2000-05-01 03:31:00
	
	debugger; 
	var bsSdtFmt;
	
	try
	{	
		bsSdtFmt = bsWmiDtFmt.substring(0, 4) + "-";  // "yyyy-"
		bsSdtFmt += bsWmiDtFmt.substr(4, 2) + DATE_SEP;  // "mm-"
		bsSdtFmt += bsWmiDtFmt.substr(6, 2) + DATE_TIME_SEP;  // "dd "
		bsSdtFmt += bsWmiDtFmt.substr(8, 2) + TIME_SEP;  // "hh:"
		bsSdtFmt += bsWmiDtFmt.substr(10, 2) + TIME_SEP;  // "mm:"
		bsSdtFmt += bsWmiDtFmt.substr(12, 2);  // "ss"

		// who cares about the rest e.g. milliseconds and timezone since
		// smallDateTime format does not account for this level of detail
		
		throw null;
	}
	
	catch(oError) 
	{
		if (null != oError) 
		{
	  		WScript.Echo(oError.number + "\n" + oError.description);
	  		return "0000-00-00 00:00:00";  // send an all zeros small date time format result
		}
		else
	  		return bsSdtFmt;
	}
}

function DateFormat(bsWmiDtFmt) 
{
	// convert yyyymmddHHMMSS.mmmmmmsUUU to mmm dd, yyyy
	// e.g.    20000501040800.000340-420 to May 04, 2000
	// if any part of the wmiDateTime value is not provided or cannot be 
	// interpreted, that portion will be replaced with today's year, month, etc.

	var oDate = ParseWmiDate(bsWmiDtFmt);
	var absMonths = new Array("January", "February", "March", "April", "May", "June", "July",
				   		      "August", "September", "October", "November", "December");

 	return absMonths[oDate.getMonth()] + " " + oDate.getDate() + ", " + oDate.getFullYear();
}

function TimeFormat(bsWmiDtFmt) 
{
	// convert yyyymmddHHMMSS.mmmmmmsUUU to hh:mm:ss
	// e.g.    20000501040800.000340-420 to 04:08:00
	// if any part of the wmiDateTime value is not provided or cannot be 
	// interpreted, that portion will be replaced with 0.

	var oTime = ParseWmiTime(bsWmiDtFmt);
	var bIsAfternoon = (oTime.getHours() > 12);
	var bsHours = (bIsAfternoon) ? oTime.getHours() - 12 : oTime.getHours();
	var bsMinutes = (oTime.getMinutes() == 0) ? "00" : oTime.getMinutes();
	var bsSeconds = (oTime.getSeconds() == 0) ? "00" : oTime.getSeconds();
	var bsAmOrPm = (bIsAfternoon) ? " PM" : " AM";

	return bsHours + TIME_SEP + bsMinutes + TIME_SEP + bsSeconds + bsAmOrPm;
}

function DateTimeFormat(bsWmiDtFmt) 
{
	// convert yyyymmddHHMMSS.mmmmmmsUUU to mmm dd, yyyy hh:mm:ss am/pm
	// e.g.    20000501040800.000340-420 to May 01, 2000 04:08:00 am
	// if any part of the wmiDateTime value is not provided or cannot be 
	// interpreted, that portion will be replaced with today's year, month, etc.

	return DateFormat(bsWmiDtFmt) + DATE_TIME_SEP + TimeFormat(bsWmiDtFmt);
}

function LocaleDateTimeFormat(bsWmiDtFmt)
{
	// convert yyyymmddHHMMSS.mmmmmmsUUU to DateTimeFormat except the
	// result is formatted according to the long date and long time 
	// formats in the regional settings.

	var oDate = ParseWmiDate(bsWmiDtFmt);
	var oTime = ParseWmiTime(bsWmiDtFmt);
	oDate.setHours(oTime.getHours());
	oDate.setMinutes(oTime.getMinutes());
	oDate.setSeconds(oTime.getSeconds());

 	return oDate.toLocaleString();
}

function ReadInt(bsValue, iBegin, iLength, iDefault) 
{
	// Wmi Date Time Format helper function
	var iRet = new Number(bsValue.substr(iBegin - 1, iLength));
	if (isNaN(iRet))
		return iDefault;
	else
		return iRet;
}

function ParseWmiDate(bsWmiDtFmt)
{
	// Wmi Date Time Format helper function
	var bsValueString = bsWmiDtFmt.toString(), oToday = new Date;
	var bsYear = ReadInt(bsValueString, 1, 4, oToday.getFullYear());
	var bsMonth = ReadInt(bsValueString, 5, 2, oToday.getMonth() + 1) - 1;
	var bsDay = ReadInt(bsValueString, 7, 2, oToday.getDate());

	return new Date(bsYear, bsMonth, bsDay);
}

function ParseWmiTime(bsWmiDtFmt)
{
	// Wmi Date Time Format helper function
	var bsValueString = bsWmiDtFmt.toString();
	var bsHour = ReadInt(bsValueString, 9, 2, 12)
	var bsMinutes = ReadInt(bsValueString, 11, 2, 0)
	var bsSeconds = ReadInt(bsValueString, 13, 2, 0)
	
	return new Date(0, 0, 0, bsHour, bsMinutes, bsSeconds);
}


