// Wep.cpp : Implementation of CWep
#include "stdafx.h"
#include "GnlPpsEvntPvdr.h"
#include "Wep.h"

/////////////////////////////////////////////////////////////////////////////
// CWep


STDMETHODIMP CWep::Initialize(LPWSTR wszUser, LONG lFlags, LPWSTR wszNamespace, LPWSTR wszLocale, IWbemServices *pNamespace, IWbemContext *pCtx, IWbemProviderInitSink *pInitSink)
{
	HRESULT hr = WBEM_NO_ERROR;  // == S_OK
	try {
		// we don't care about most of the incoming parameters in this service.  We only need to save the namespace 
		// pointer, get our event class definition and return the initialized status.  Using smart pointer member 
		// variables to encapsulate the raw interface pointers will call AddRef and Release for us automatically
		_spNamespace.Attach(pNamespace, TRUE);

		// grab the class definition for the event
		hr = _spNamespace->GetObject(_bstr_t(EVENT_CLASS), 0, pCtx, &_spEventClassDef, NULL);
		// if (FAILED(hr)) { throw hr; }
		if (FAILED(hr)) { hr = WBEM_E_FAILED; throw hr; }
		// or possible approach for supporting support multiple event types without having to dig into pCtx
		/*
		hr = _spNamespace->GetObject(_bstr_t(CREATION_EVENT_CLASS), 0, pCtx, &_spEventClassDef, NULL);
		if (FAILED(hr) && 0x80041002 != hr) { hr = WBEM_E_FAILED; throw hr; }
		hr = _spNamespace->GetObject(_bstr_t(TERMINATION_EVENT_CLASS), 0, pCtx, &_spEventClassDef, NULL);
		if (FAILED(hr)) { hr = WBEM_E_FAILED; throw hr; }
		*/

		// note - [optionally] use an instance provider to create an instance of a GpepApplication class and submit that
		// as an event class TargetInstance object property therefore matching the behavior of most wmi intrinsic events

		// tell CIMOM that we are up and running, aka initialized
		hr = pInitSink->SetStatus(WBEM_S_INITIALIZED, 0);
		if (FAILED(hr)) throw hr;

		hr = WBEM_S_NO_ERROR;  // == WBEM_NO_ERROR == S_OK
	}

	catch(...) {
		OutputDebugStringEx(hr, _bstr_t(_T("Initialize")));
		if (SUCCEEDED(hr)) throw;  // pass unexpected exceptions on to the outer block
	}
    
	return hr;
}


STDMETHODIMP CWep::ProvideEvents(IWbemObjectSink *pSink, long lFlags)
{
	HRESULT hr = WBEM_NO_ERROR;  // == S_OK
	try {
		// copy the sink
		_spSink.Attach(pSink, TRUE);  // AddRef is called

		/*
		// psdk sample local event generation thread procedures, see <psdk>\samples\sysmgmt\wmi\vc\eventProvider
		// commented out for production use of provider

		// create the event thread
		UINT uiThreadId = 0;

		// note - defining threadproc as a class method even if defined and called as a static versus instance method 
		// doesn't want to cast properly so we will define and call the threadproc as a standard winapi function
		// _hThread = (void *)_beginthreadex(NULL, 0, CWep::EventThread, this, 0, &uiThreadId);
		// _hThread = (void *)_beginthreadex(NULL, 0, this->EventThread, this, 0, &uiThreadId);
		_hThread = (void *)_beginthreadex(NULL, 0, EventThread, this, 0, &uiThreadId);

		// wait for provider to be running before returning status
		while (_eWepStatus != Running)
			Sleep(100);
		*/

		_eWepStatus = Running;  // set status to running since thread routine will not be doing this

		hr = WBEM_NO_ERROR;  // == S_OK
	}

	catch(...) {
		OutputDebugStringEx(hr, _bstr_t(_T("ProvideEvents")));
		if (SUCCEEDED(hr)) throw;  // pass unexpected exceptions on to the outer block
	}
    
	return hr;
}


UINT WINAPI EventThread(LPVOID pArg)
// STDMETHODIMP CWep::EventThread(/* LPVOID */ LONG *pArg)
{
	// this particular provider, being in a DLL operates via its own thread.  In practice, such a provider would probably
	// be implemented within a separate EXE - ???
	HRESULT hr = S_OK;

	try {
		// validate parameters
		if (!pArg) { hr = E_INVALIDARG; throw hr; }
				
		// make transition to the per-instance method
		// ((CWep *)pArg)->InstanceThread();
		/*
		hr = ((CWep *)pArg)->InstanceThread(_bstr_t("test.js"), _bstr_t("myUser@mydomain.com"), 1001,
											_bstr_t("c:\\path\\test.js"), _variant_t("\0"),
											_variant_t("\0"),
											_variant_t("\0"));
		*/
		hr = ((CWep *)pArg)->InstanceThread(_bstr_t("test.js"), _bstr_t("myUser@mydomain.com"), 1001,
											_bstr_t("c:\\path\\test.js"), _variant_t("myhost"),
											_variant_t("20010110034230.000042-480"),
											_variant_t(""));
	}

	catch(...) {
		OutputDebugStringEx(hr, _bstr_t(_T("Fire")));
		// if (SUCCEEDED(hr)) throw;  // [optionally] pass unexpected exceptions on to the outer block
	}
    
	return hr;
}


STDMETHODIMP CWep::InstanceThread(BSTR bsApplication, BSTR bsUser, INT iNumberValue, BSTR bsStringValue, VARIANT vHostname, VARIANT vCreationDate, VARIANT vTerminationDate)
{
	// events are generated from here, that or this thread holds wbem | consumer sink reference to object open
	// thus making it possible to fire events
	HRESULT hr = S_OK;
	try {
		// validate parameters
		if (!bsApplication || !SysStringLen(bsApplication) ||
			!bsUser || !SysStringLen(bsUser) ||
			/* !iNumberValue || */
			!bsStringValue || !SysStringLen(bsStringValue) // || 
			/* !vHostname || !vCreationDate || !vTerminationDate */) { hr = E_INVALIDARG; throw hr; }

		INT iIteration = 0;

		_eWepStatus = Running;
        
		// now check to see if any consumers have registered to receive the general purpose event provider events and if
		// not just exit right away in the interest of efficiency
		// TODO: select an better wbem hresult code to return or define a custom one for this case
		// if (Running != m_eWepStatus /* || NULL == m_spEventClassDef || NULL == m_spSink */) { /* hr = E_FAIL; */ throw hr; }

		while (Running == _eWepStatus) {
			Sleep(2000);  // provide an event every two seconds

			// generate a new event object
			IWbemClassObject *pEvent = 0;
			//IWbemClassObjectPtr spEvent;        
			
			hr = _spEventClassDef->SpawnInstance(0, &pEvent);
			//hr = m_spEventClassDef->SpawnInstance(0, &spEvent);
			if (FAILED(hr)) throw hr;

			// note - [optionally] use an instance provider to create an instance of a GpepApplication class and submit that
			// as an event class TargetInstance object property therefore matching the behavior of most wmi intrinsic events

			// use the passed values to populate the event
			_variant_t vTemp(bsApplication);
			hr = pEvent->Put(_T("Application"), 0, &vTemp, 0);
			//hr = pEvent->Put(_T("Application"), 0, &vTemp, VT_BSTR);
			//hr = spEvent->Put(_T("Application"), 0, &vTemp, VT_BSTR);
			if (FAILED(hr)) throw hr;

			vTemp.Clear();
			vTemp = bsUser;
			hr = pEvent->Put(_T("User"), 0, &vTemp, 0);
			//hr = pEvent->Put(_T("User"), 0, &vTemp, VT_BSTR);
			//hr = spEvent->Put(_T("User"), 0, &vTemp, VT_BSTR);
			if (FAILED(hr)) throw hr;

			vTemp.Clear(); 
			vTemp = (LONG)iNumberValue;
			hr = pEvent->Put(_T("NumberValue"), 0, &vTemp, 0);
			//hr = pEvent->Put(_T("NumberValue"), 0, &vTemp, VT_I4);
			//hr = spEvent->Put(_T("NumberValue"), 0, &vTemp, VT_I4);
			if (FAILED(hr)) throw hr;

			vTemp.Clear();
			vTemp = bsStringValue;
			hr = pEvent->Put(_T("StringValue"), 0, &vTemp, 0);
			//hr = pEvent->Put(_T("StringValue"), 0, &vTemp, VT_BSTR);
			//hr = spEvent->Put(_T("StringValue"), 0, &vTemp, VT_BSTR);
			if (FAILED(hr)) throw hr;

			// TODO: resolve best way to autogenerate remaining three optional parameters if not provided
			// note - VARIANT.vt == VT_ERROR results from optional automation method argument left blank
			if (VT_ERROR != vHostname.vt) {
				hr = pEvent->Put(_T("Hostname"), 0, &vHostname, 0);
				//hr = pEvent->Put(_T("Hostname"), 0, &vHostname, VT_BSTR);
				//hr = spEvent->Put(_T("Hostname"), 0, &vHostname, VT_BSTR);
				if (FAILED(hr)) throw hr;
			}

			#if defined(_DEBUG)
				INT iTest = 0;
				if (VT_ERROR != vCreationDate.vt) 
					iTest = _tcslen(vCreationDate.bstrVal);
				iTest = 0;
				if (VT_ERROR != vTerminationDate.vt) 
					iTest = _tcslen(vTerminationDate.bstrVal);
			#endif

			if ((VT_ERROR != vCreationDate.vt) && _tcslen(vCreationDate.bstrVal)) {
				hr = pEvent->Put(_T("CreationDate"), 0, &vCreationDate, 0);
				//hr = pEvent->Put(_T("CreationDate"), 0, &vCreationDate, VT_BSTR);
				//hr = spEvent->Put(_T("CreationDate"), 0, &vCreationDate, VT_BSTR);
				if (FAILED(hr)) throw hr;
			}

			if ((VT_ERROR != vTerminationDate.vt) && _tcslen(vTerminationDate.bstrVal)) {
				hr = pEvent->Put(_T("TerminationDate"), 0, &vTerminationDate, 0);
				//hr = pEvent->Put(_T("TerminationDate"), 0, &vTerminationDate, VT_BSTR);
				//hr = spEvent->Put(_T("TerminationDate"), 0, &vTerminationDate, VT_BSTR);
				if (FAILED(hr)) throw hr;
			}

			// deliver the event to CIMOM
			hr = _spSink->Indicate(1, &pEvent);
			//hr = m_spSink->Indicate(1, &spEvent);
			if (FAILED(hr)) { /* throw hr; */ /* if here, delivery failed.  Do something to report it */ }
			// -2147217402/0x80041006 == WBEM_E_OUT_OF_MEMORY

			pEvent->Release();
			// if (pEvent) pEvent->Release(); need a catch for this when throws occur ???

			iIteration++;
		}

		// when we get to here, we are no longer interested in the provider and Release() has long since returned
		_eWepStatus = Stopped;
		// delete this;  with cause exception w/o does not ???
	}

	catch(...) {
		OutputDebugStringEx(hr, _bstr_t(_T("Fire")));
		// if (SUCCEEDED(hr)) throw;  // [optionally] pass unexpected exceptions on to the outer block
	}
    
	return hr;
}

